function [R0p,M0p,Cp,R,Ms,Mz,Rp,Msp,Mzp,Rs,Mss,Mzs]=...
    generate_test_data_uncertain_frames(sigma_g,sigma_n,N_samples)
%%  generate test data for checking uncertain frames
%
% sigma_g       stdv. for generating frames
% sigma_n       stdv. for generaing noise
% N_samples     number of samples for checking CovMs
%
% R0p   [3x6] rotation pair [R1,R2]
% M0p   [4x8] motion pair [M1,M2]
% Cp    [12x12] CovM, also used partially 
% R     uncertain rotation, R.R=[3x3], R.C=[3x3]
% Ms    uncertain motion_s, Ms.Ms=[4x4], Ms.Cs=[6x6]
% Mz    uncertain motion_zeta, Mz.Mz=[4x4], Mz.Cz=[6x6]
% Rp    uncertain rotation pair, Rp.Rp=[R1,R2]=[3x6], Rp.Cp=[6x6]
% Msp   uncertain motion pair s, Msp.Msp=[Ms1,Ms2]=[4x8], Msp.Csp=[12x12]
% Mzp   uncertain motion pair zeta, Mzp.Mzp=[Mz1,Mz2]=[4x8], Mzp.Czp=[12x12]
% Rs    [9xN_samples] noisy vectorized rotation matrices
% Mss   [16xN_samples] noisy vectorized motion matrices s
% Mzs   [16xN_samples] noisy vectorized motion matrices zeta
%
% wf 5/2020

%% generate covariance matrix 12x12
% will be used throughout
B  = randn(12);
Cp = B'*B*sigma_n^2;

%% rotations
% two rotations
theta0  = randn(6,1)*sigma_g;
R01    = expm(calc_S(theta0(1:3)));
R02    = expm(calc_S(theta0(4:6)));
R0p    = [R01,R02];

% uncertain rotations
r = mvnrnd(zeros(6,1),Cp(1:6,1:6),1)';
S1 = calc_S(r(1:3));  % S = r_wedge
R1 = expm(S1)*R01;
S2 = calc_S(r(4:6));  % S = r_wedge
R2 = expm(S2)*R02;

R.R = R1;
R.C = Cp(1:3,1:3);
Rp.Rp = [R1,R2];
Rp.Cp = Cp(1:6,1:6);

%% motions
% motions
s0    = randn(12,1)*sigma_g;
M01    = expm(calc_A_from_s(s0(1:6)));
M02    = expm(calc_A_from_s(s0(7:12)));
M0p    = [M01,M02];

% Uncertain motions s

s = mvnrnd(zeros(12,1),Cp,1)';
M1 = expm(calc_A_from_s(s(1:6)))*M01;
M2 = expm(calc_A_from_s(s(7:12)))*M02;

% single motion
Ms.Ms = M1;
Ms.Cs = Cp(1:6,1:6);
% motion pair
Msp.Msp = [M1,M2];
Msp.Csp = Cp;

% uncertain motions zeta

zeta = mvnrnd(zeros(12,1),Cp,1)';
M1 = [expm(calc_S(zeta(1:3)))*M01(1:3,1:3) M01(1:3,4)+zeta(4:6); 0 0 0 1];
M2 = [expm(calc_S(zeta(7:9)))*M02(1:3,1:3) M02(1:3,4)+zeta(10:12); 0 0 0 1];

% single motion
Mz.Mz = M1;
Mz.Cz = Cp(1:6,1:6);
% motion pair
Mzp.Mzp = [M1,M2];
Mzp.Czp = Cp;

%% samples

% rotations
rs = mvnrnd(zeros(3,1),Cp(1:3,1:3),N_samples)';
check_CovM(cov(rs'),Cp(1:3,1:3),N_samples,0.99);
Rs = zeros(9,N_samples);
for n = 1:N_samples
    Rn      = expm(calc_S(rs(:,n)))*R01;
    Rs(:,n) = Rn(:);
end

% motions s
ss = mvnrnd(zeros(6,1),Cp(1:6,1:6),N_samples)';
Mss = zeros(16,N_samples);
for n=1:N_samples
    Mn = expm(calc_A_from_s(ss(:,n)))*M01;
    Mss(:,n)=Mn(:);
end

% motions zeta
zetas = mvnrnd(zeros(6,1),Cp(1:6,1:6),N_samples)';
Mzs = zeros(16,N_samples);
for n=1:N_samples
    Mn = [expm(calc_S(zetas(1:3,n)))*M01(1:3,1:3) M01(1:3,4)+zetas(4:6,n); ...
        0 0 0 1];
    Mzs(:,n)=Mn(:);
end


return
